package compbio.cassandra;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.EOFException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import compbio.cassandra.JpredParser;

public class JpredParserHTTP implements JpredParser {
	private CassandraWriter cw = new CassandraWriter();
	private String dirprefix;
	private List<FastaSequence> alignment;
	private List<FastaSequence> predictions;
	private int countNoData;

	public JpredParserHTTP() {
		dirprefix = "http://www.compbio.dundee.ac.uk/www-jpred/results";
	}

	public JpredParserHTTP(String sourceurl) {
		dirprefix = sourceurl;
	}

	public void setSource(String newsourceprefix) {
		dirprefix = newsourceprefix;
	}

	public void Parsing(String source, int nDays) throws IOException {
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -nDays);
		for (int i = 0; i < nDays; ++i) {
			cal.add(Calendar.DATE, 1);
			int month = cal.get(Calendar.MONTH) + 1;
			int year = cal.get(Calendar.YEAR);
			int day = cal.get(Calendar.DATE);
			String date = year + "/" + month + "/" + day;
			ParsingForDate(source, date);
		}
	}

	/*
	 * The method parses the Jpred output concise file in the FASTA format If
	 * there is a record with ID = QUERY or jobid, this a "one protein" job
	 * otherwise this is an alignment job
	 */
	private String parsePredictions(final InputStream stream, String jobid) throws FileNotFoundException {
		final FastaReader fr = new FastaReader(stream);
		String protein = "";
		alignment = new ArrayList<FastaSequence>();
		predictions = new ArrayList<FastaSequence>();
		while (fr.hasNext()) {
			final FastaSequence fs = fr.next();
			String seqid = fs.getId();
			String seq = fs.getSequence().replaceAll("\n", "");
			if (seqid.equals("jnetpred") || seqid.equals("Lupas_21") || seqid.equals("Lupas_14") || seqid.equals("Lupas_28")
					|| seqid.equals("JNETSOL25") || seqid.equals("JNETSOL5") || seqid.equals("JNETSOL0") || seqid.equals("JNETCONF")
					|| seqid.equals("JNETHMM") || seqid.equals("JNETPSSM") || seqid.equals("JNETCONF")) {
				predictions.add(fs);
			} else {
				alignment.add(fs);
				if (seqid.equals("QUERY") || seqid.equals(jobid))
					protein = seq;
			}
		}
		return protein;
	}

	private String parseLogFile(final InputStream stream) throws IOException {
		String out = "";
		BufferedReader buffer = new BufferedReader(new InputStreamReader(stream));
		String line;
		while (null != (line = buffer.readLine())) {
			out += line;
		}
		return out;
	}

	private List<Byte> parseArchiveFile(final InputStream stream) throws IOException {
		DataInputStream data_in = new DataInputStream(stream);
		List<Byte> out = new ArrayList<Byte>();
		while (true) {
			try {
				out.add(data_in.readByte());
			} catch (EOFException eof) {
				break;
			}
		}
		return out;
	}

	private int analyseJob(String[] job) throws IOException {
		boolean running = true;
		boolean ConcisefileExists = false;
		boolean LogfileExists = false;
		String id = job[job.length - 1];
		String startdatestring = job[0].substring(0, job[0].indexOf(":"));
		Date startdate = new Date(0);
		Date starttime = new Date(0);
		Date endtime = new Date(0);
		Date currDate = new Date();
		String ip = job[2];
		String execstatus = "OK";
		String finalstatus = "OK";
		String protein = "";
		long exectime = 0;
		String log = "";
		String maindir = dirprefix + "/" + id + "/";
		String concisefile = dirprefix + "/" + id + "/" + id + ".concise.fasta";
		String archivefile = dirprefix + "/" + id + "/" + id + ".tar.gz";
		String logfile = dirprefix + "/" + id + "/LOG";
		SimpleDateFormat dateformatter = new SimpleDateFormat("yyyy/MM/dd");
		SimpleDateFormat timeformatter = new SimpleDateFormat("yyyy/MM/dd:H:m:s");
		try {
			startdate = dateformatter.parse(startdatestring);
			starttime = timeformatter.parse(job[0]);
			endtime = timeformatter.parse(job[1]);
			exectime = (endtime.getTime() - starttime.getTime()) / 1000;
		} catch (ParseException e) {
			e.printStackTrace();
		}

		try {
			URL dirurl = new URL(maindir);
			HttpURLConnection httpConnection_dirurl = (HttpURLConnection) dirurl.openConnection();
			if (httpConnection_dirurl.getResponseCode() < 199 || 300 <= httpConnection_dirurl.getResponseCode()) {
				return 0;
			}
			URL conciseurl = new URL(concisefile);
			URL archiveurl = new URL(archivefile);
			URL logurl = new URL(logfile);
			HttpURLConnection httpConnection_conciseurl = (HttpURLConnection) conciseurl.openConnection();
			HttpURLConnection httpConnection_logurl = (HttpURLConnection) logurl.openConnection();
			HttpURLConnection httpConnection_archiveurl = (HttpURLConnection) archiveurl.openConnection();
			if (199 < httpConnection_conciseurl.getResponseCode() && httpConnection_conciseurl.getResponseCode() < 300) {
				ConcisefileExists = true;
				running = false;
				try {
					protein = parsePredictions(conciseurl.openStream(), id);
				} catch (IOException e) {
					e.printStackTrace();
				}
			} else {
				// The job still can be running of failed...
				++countNoData;
				alignment = new ArrayList<FastaSequence>();
				predictions = new ArrayList<FastaSequence>();
			}
			if (199 < httpConnection_logurl.getResponseCode() && httpConnection_logurl.getResponseCode() < 300) {
				LogfileExists = true;
				log = parseLogFile(logurl.openStream());
			} else {
				// The job has not been started at all...
				execstatus = "FAIL";
				finalstatus = "STOPPED";
				running = false;
			}
			if (log.matches("(.*)TIMEOUT\\syour\\sjob\\stimed\\sout(.*)")) {
				// blast job was too long (more than 3600 secs by default)...
				execstatus = "FAIL";
				finalstatus = "TIMEDOUT";
				running = false;
			} else if (log.matches("(.*)Jpred\\serror:\\sDied\\sat(.*)")) {
				// an internal Jpred error...
				execstatus = "FAIL";
				finalstatus = "JPREDERROR";
				running = false;
			} else if ((currDate.getTime() - endtime.getTime()) / 1000 > 3601 && LogfileExists && !ConcisefileExists) {
				// the job was stopped with unknown reason...
				execstatus = "FAIL";
				finalstatus = "STOPPED";
				running = false;
			}

			httpConnection_conciseurl.disconnect();
			httpConnection_logurl.disconnect();
			httpConnection_archiveurl.disconnect();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}

		if (!running) {
			long t = startdate.getTime();
			cw.FormQueryTables(t, job[0], job[1], ip, id, execstatus, finalstatus, protein, predictions);
			cw.ArchiveData(t, exectime, ip, id, execstatus, finalstatus, protein, predictions, alignment, log, archivefile);
			return 1;
		} else
			System.out.println("job " + id + " is running");

		return 0;
	}

	private void ParsingForDate(String input, String date) {
		int totalcount = 0;
		int countinsertions = 0;
		int countinserted = 0;
		int countNotanalyzed = 0;
		countNoData = 0;

		System.out.println("Inserting jobs for " + date);
		try {
			URL url = new URL(input);
			URLConnection conn = url.openConnection();
			BufferedReader alljobs = new BufferedReader(new InputStreamReader(conn.getInputStream()));
			String line;

			while ((line = alljobs.readLine()) != null) {
				if (line.matches(date + ":(.*)jp_[^\\s]+")) {
					totalcount++;
					String[] job = line.split("\\s+");
					String jobid = job[job.length - 1];
					if (cw.JobisNotInsterted(jobid)) {
						countinsertions += analyseJob(job);
					} else {
						++countinserted;
					}
				} else {
					++countNotanalyzed;
				}
			}
			alljobs.close();
			System.out.println("Total number of jobs = " + totalcount);
			System.out.println("   " + countinserted + " jobs inserted already");
			System.out.println("   " + countNotanalyzed + " not analysed jobs");
			System.out.println("   " + countNoData + " jobs without *.concise.fasta file (RUNNING or FAILED)");
			System.out.println("   " + countinsertions + " new job insertions\n");
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		;
	}
}
